@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc.h,v 1.19 2005/06/03 03:49:58 daniel Exp $
+ * $Id: avc.h,v 1.24 2006/08/17 20:09:45 dmorissette Exp $
  *
  * Name:     avc.h
  * Project:  Arc/Info Vector coverage (AVC) BIN<->E00 conversion library
@@ -30,6 +30,24 @@
  **********************************************************************
  *
  * $Log: avc.h,v $
+ * Revision 1.24  2006/08/17 20:09:45  dmorissette
+ * Update for 2.0.0 release
+ *
+ * Revision 1.23  2006/08/17 18:56:42  dmorissette
+ * Support for reading standalone info tables (just tables, no coverage
+ * data) by pointing AVCE00ReadOpen() to the info directory (bug 1549).
+ *
+ * Revision 1.22  2006/06/27 18:38:43  dmorissette
+ * Cleaned up E00 reading (bug 1497, patch from James F.)
+ *
+ * Revision 1.21  2006/06/16 11:48:11  daniel
+ * New functions to read E00 files directly as opposed to translating to
+ * binary coverage. Used in the implementation of E00 read support in OGR.
+ * Contributed by James E. Flemer. (bug 1497)
+ *
+ * Revision 1.20  2006/06/14 16:31:28  daniel
+ * Added support for AVCCoverPC2 type (bug 1491)
+ *
  * Revision 1.19  2005/06/03 03:49:58  daniel
  * Update email address, website url, and copyright dates
  *
@@ -104,7 +122,7 @@ CPL_C_START
 /*---------------------------------------------------------------------
  * Current version of the AVCE00 library... always useful!
  *--------------------------------------------------------------------*/
-#define AVC_VERSION "1.3.0 (2005-06-02)"
+#define AVC_VERSION "2.0.0 (2006-08-17)"
 
 
 /* Coverage precision
@@ -153,7 +171,9 @@ typedef enum
     AVCCoverTypeUnknown = 0,
     AVCCoverV7,
     AVCCoverPC,
-    AVCCoverWeird  /* Unknown version... hybrid between V7 and PC !!! */
+    AVCCoverPC2,   /* Unknown version... hybrid between V7 and PC !!! */
+    AVCCoverWeird, /* Unknown version... hybrid between V7 and PC !!! */
+    AVCCoverV7Tables /* Standalone tables, only an info directory     */
 } AVCCoverType;
 
 /* Enum for byte ordering
@@ -167,10 +187,10 @@ typedef enum
 
 /* Macros to establish byte ordering for each coverage type
  * The rule until now: all coverage types use big endian (Motorola ordering)
- * except PC Arc/Info coverages.
+ * except PC Arc/Info coverages variant 1 (AVCCoverPC).
  */
 #define AVC_COVER_BYTE_ORDER(cover_type)  \
-   ((cover_type) == AVCCoverPC ? AVCLittleEndian : AVCBigEndian )
+   (((cover_type) == AVCCoverPC ) ? AVCLittleEndian : AVCBigEndian )
 
 /*=====================================================================
                         Structures
@@ -409,7 +429,7 @@ typedef struct AVCBinFile_t
     char          *pszFilename;
     AVCRawBinFile *psIndexFile;   /* Index file, Write mode only */
 
-    DBFHandle     hDBFFile;       /* Used for AVCCoverPC DBF TABLES only */
+    DBFHandle     hDBFFile;       /* Used for AVCCoverPC/PC2 DBF TABLES only */
     int           nCurDBFRecord;  /* 0-based record index in DBF file */
 
     AVCCoverType  eCoverType;
@@ -476,6 +496,8 @@ typedef struct AVCE00ParseInfo_t
     int         nPrecision;     /* AVC_SINGLE/DOUBLE_PREC       */
     int         iCurItem;
     int         numItems;
+    int         nStartLineNum;
+    int         nCurLineNum;
 
     int         nCurObjectId;
     GBool       bForceEndOfSection;  /* For sections that don't have an */
@@ -519,7 +541,9 @@ typedef struct AVCE00Section_t
 {
     AVCFileType eType;        /* File Type                      */
     char        *pszName;     /* E00 section or Table Name      */
-    char        *pszFilename; /* Binary file filename           */
+    char        *pszFilename; /* Binary/E00 file filename       */
+    int         nLineNum;     /* E00 line number                */
+    int         nFeatureCount;
 }AVCE00Section;
 
 typedef struct AVCE00ReadInfo_t
@@ -556,6 +580,34 @@ typedef struct AVCE00ReadInfo_t
 
 } *AVCE00ReadPtr;
 
+typedef struct AVCE00ReadInfoE00_t
+{
+    char        *pszCoverPath;
+    char        *pszCoverName;
+
+    AVCE00ParseInfo *hParseInfo;
+    AVCFileType  eCurFileType;
+
+    /* pasSections is built when the coverage is opened and describes
+     * the sections in the E00 file.
+     */
+    AVCE00Section *pasSections;
+    int            numSections;
+
+    /* If bReadAllSections=TRUE then reading automatically continues to
+     * the next section when a section finishes. (This is the default)
+     * Otherwise, you can use AVCE00ReadGotoSectionE00() to read one
+     * section at a time.
+     */ 
+    GBool         bReadAllSections;
+
+    /* File handle of the E00 file currently being processed
+     */
+    FILE          *hFile;
+
+} *AVCE00ReadE00Ptr;
+
+
 /* E00 generation steps... tells the AVCE00Read*() functions which
  * parts of the given E00 file are currently being processed.
  */
@@ -769,6 +821,19 @@ int  AVCPrintRealValue(char *pszBuf, int nPrecision, AVCFileType eType,
               Function prototypes (THE PUBLIC ONES)
  =====================================================================*/
 
+/*---------------------------------------------------------------------
+ * Functions to read E00
+ *--------------------------------------------------------------------*/
+AVCE00ReadE00Ptr AVCE00ReadOpenE00(const char *pszE00FileName);
+void             AVCE00ReadCloseE00(AVCE00ReadE00Ptr psRead);
+int              AVCE00ReadRewindE00(AVCE00ReadE00Ptr psRead);
+void            *AVCE00ReadNextObjectE00(AVCE00ReadE00Ptr psRead);
+
+AVCE00Section  *AVCE00ReadSectionsListE00(AVCE00ReadE00Ptr psRead, int *numSect);
+int             AVCE00ReadGotoSectionE00(AVCE00ReadE00Ptr psRead, 
+                                         AVCE00Section *psSect,
+                                         GBool bContinue);
+
 /*---------------------------------------------------------------------
  * Functions to make a binary coverage appear as E00
  *--------------------------------------------------------------------*/
