@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc_bin.c,v 1.27 2005/06/03 03:49:58 daniel Exp $
+ * $Id: avc_bin.c,v 1.29 2006/08/17 18:56:42 dmorissette Exp $
  *
  * Name:     avc_bin.c
  * Project:  Arc/Info vector coverage (AVC)  BIN->E00 conversion library
@@ -30,6 +30,13 @@
  **********************************************************************
  *
  * $Log: avc_bin.c,v $
+ * Revision 1.29  2006/08/17 18:56:42  dmorissette
+ * Support for reading standalone info tables (just tables, no coverage
+ * data) by pointing AVCE00ReadOpen() to the info directory (bug 1549).
+ *
+ * Revision 1.28  2006/06/14 16:31:28  daniel
+ * Added support for AVCCoverPC2 type (bug 1491)
+ *
  * Revision 1.27  2005/06/03 03:49:58  daniel
  * Update email address, website url, and copyright dates
  *
@@ -177,7 +184,7 @@ AVCBinFile *AVCBinReadOpen(const char *pszPath, const char *pszName,
      *----------------------------------------------------------------*/
     if (eFileType == AVCFileTABLE)
     {
-        if (eCoverType == AVCCoverPC)
+        if (eCoverType == AVCCoverPC || eCoverType == AVCCoverPC2)
             return _AVCBinReadOpenDBFTable(pszPath, pszName);
         else
             return _AVCBinReadOpenTable(pszPath, pszName, 
@@ -375,8 +382,9 @@ int _AVCBinReadHeader(AVCRawBinFile *psFile, AVCBinHeader *psHeader,
     int nStatus = 0;
 
     /*-----------------------------------------------------------------
-     * For AVCCoverPC coverages, there is a first 256 bytes header
-     * that we just skip and that precedes the 100 bytes header block.
+     * For AVCCoverPC coverages (files without hte .adf extension), 
+     * there is a first 256 bytes header that we just skip and that 
+     * precedes the 100 bytes header block.
      *
      * In AVCCoverV7, we only have the 100 bytes header.
      *----------------------------------------------------------------*/
@@ -434,7 +442,8 @@ int AVCBinReadRewind(AVCBinFile *psFile)
      * For AVCCoverPC coverages, there is a first 256 bytes header
      * that we just skip and that precedes the 100 bytes header block.
      *
-     * In AVCCoverV7 and AVCCoverWeird, we only find the 100 bytes header.
+     * In AVCCoverV7, AVCCoverPC2 and AVCCoverWeird, we only find the 
+     * 100 bytes header.
      *
      * Note: it is the call to _AVCBinReadHeader() that takes care
      * of skipping the first 256 bytes header if necessary.
@@ -716,6 +725,7 @@ void *AVCBinReadNextObject(AVCBinFile *psFile)
 AVCField *AVCBinReadNextTableRec(AVCBinFile *psFile)
 {
     if (psFile->eCoverType != AVCCoverPC &&
+        psFile->eCoverType != AVCCoverPC2 &&
         psFile->eFileType == AVCFileTABLE &&
         psFile->hdr.psTableDef->numRecords > 0 &&
         ! AVCRawBinEOF(psFile->psRawBinFile) &&
@@ -727,7 +737,8 @@ AVCField *AVCBinReadNextTableRec(AVCBinFile *psFile)
     {
         return psFile->cur.pasFields;
     }
-    else if (psFile->eCoverType == AVCCoverPC &&
+    else if ((psFile->eCoverType == AVCCoverPC ||
+              psFile->eCoverType == AVCCoverPC2 ) &&
              psFile->eFileType == AVCFileTABLE &&
              psFile->hdr.psTableDef->numRecords > 0 &&
              _AVCBinReadNextDBFTableRec(psFile->hDBFFile, 
@@ -1790,6 +1801,13 @@ char **AVCBinReadListTables(const char *pszInfoPath, const char *pszCoverName,
     if (ppapszArcDatFiles)
         *ppapszArcDatFiles = NULL;
 
+    /*----------------------------------------------------------------- 
+     * For AVCCoverV7Tables type we do not look for tables for a specific
+     * coverage, we return all tables from the info dir.
+     *----------------------------------------------------------------*/
+    if (eCoverType == AVCCoverV7Tables)
+        pszCoverName = NULL;
+
     /*----------------------------------------------------------------- 
      * All tables that belong to a given coverage have their name starting
      * with the coverage name (in uppercase letters), followed by a 3 
