@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: avc_e00gen.c,v 1.16 2005/06/03 03:49:58 daniel Exp $
+ * $Id: avc_e00gen.c,v 1.17 2006/06/14 15:01:33 daniel Exp $
  *
  * Name:     avc_e00gen.c
  * Project:  Arc/Info vector coverage (AVC)  BIN->E00 conversion library
@@ -30,6 +30,9 @@
  **********************************************************************
  *
  * $Log: avc_e00gen.c,v $
+ * Revision 1.17  2006/06/14 15:01:33  daniel
+ * Remove any embeded '\0' from data line in AVCE00GenTableRec()
+ *
  * Revision 1.16  2005/06/03 03:49:58  daniel
  * Update email address, website url, and copyright dates
  *
@@ -1307,7 +1310,7 @@ const char *AVCE00GenTableRec(AVCE00GenInfo *psInfo, int numFields,
             if (nType ==  AVC_FT_DATE || nType == AVC_FT_CHAR ||
                 nType == AVC_FT_FIXINT )
             {
-                strncpy(pszBuf2, pasFields[i].pszStr, nSize);
+                memcpy(pszBuf2, pasFields[i].pszStr, nSize * sizeof(char));
                 pszBuf2 += nSize;
             }
 #ifdef AVC_MAP_TYPE40_TO_DOUBLE
@@ -1386,6 +1389,18 @@ const char *AVCE00GenTableRec(AVCE00GenInfo *psInfo, int numFields,
         }
 
         *pszBuf2 = '\0';
+
+	/* Make sure that we remove any embedded NUL characters from the
+	 * data line before returning it, otherwise we may be accidentally
+	 * truncating results.
+	 */
+	while ( --pszBuf2 >= psInfo->pszBuf+81 )
+	{
+            if ( *pszBuf2 == '\0' )
+            {
+                *pszBuf2 = ' ';
+            }
+	}
     }
 
     if (psInfo->iCurItem < psInfo->numItems)
